"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

exports.wrap = wrap;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _index = require("./index");

var _cssColorNames = require("css-color-names");

var _cssColorNames2 = _interopRequireDefault(_cssColorNames);

var _lineHeight = require("./styles/lineHeight");

var _lineHeight2 = _interopRequireDefault(_lineHeight);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * Wrap takes a Component or a render function and recursively replaces
 * the prop 'cls' (or custom overridden prop name) with the respective 'style' definitions.
 * Render functions are decorated so that when they are invoked the resulting
 * element tree will be be recursively evaluated and 'cls' will be replaced with
 * 'style' definitions.
 */
function wrap(componentOrFunction) {
    if (!(componentOrFunction.prototype && "render" in componentOrFunction.prototype)) {
        var func = componentOrFunction;

        return function wrappedRender() {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            /* eslint-disable no-invalid-this */
            return recursiveStyle(func.apply(this, args));
        };
    }
    var WrappedComponent = componentOrFunction;
    var newClass = function (_WrappedComponent) {
        _inherits(newClass, _WrappedComponent);

        function newClass() {
            _classCallCheck(this, newClass);

            return _possibleConstructorReturn(this, (newClass.__proto__ || Object.getPrototypeOf(newClass)).apply(this, arguments));
        }

        _createClass(newClass, [{
            key: "render",
            value: function render() {
                return recursiveStyle(_get(newClass.prototype.__proto__ || Object.getPrototypeOf(newClass.prototype), "render", this).call(this));
            }
        }]);

        return newClass;
    }(WrappedComponent);

    /* Fix name */
    newClass.displayName = WrappedComponent.displayName || WrappedComponent.name;

    /* Mark the class as wrapped by tachyons */
    newClass.isTachyonsWrapped = true;

    return newClass;
}

function setStyles(props, clsPropName, typeScale) {
    var newProps = {};
    if (_lodash2.default.isArray(props.style)) {
        newProps.style = props.style.slice();
    } else if (_lodash2.default.isObject(props.style)) {
        newProps.style = [props.style];
    } else {
        newProps.style = [];
    }

    var splitted = props[clsPropName].replace(/\x2D/g, "_").split(" ");
    var fontSize = _lodash2.default.find(_lodash2.default.keys(typeScale), function (fSetting) {
        return _lodash2.default.includes(splitted, fSetting);
    });

    for (var i = 0; i < splitted.length; i++) {
        var cls = splitted[i];
        if (cls.length > 0) {
            if (_index.styles[cls]) {

                /* Style found */
                newProps.style.push(_index.styles[cls]);
            } else if (cls.startsWith("lh_")) {

                /* Get font style */
                if (!_lodash2.default.isString(fontSize)) {
                    throw new Error("setting '" + cls + "' needs explicit font-size");
                }

                newProps.style.push({
                    lineHeight: _lineHeight2.default[cls.replace(/_/g, "-")] * _index.styles[fontSize].fontSize
                });
            } else if (cls.startsWith("bg_")) {
                newProps.style.push({
                    backgroundColor: cls.slice(3)
                });
            } else if (cls.startsWith("b__")) {
                newProps.style.push({
                    borderColor: cls.slice(3)
                });
            } else if (cls.startsWith("tint_")) {
                newProps.style.push({
                    tintColor: cls.slice(3)
                });
            } else if (_cssColorNames2.default[cls] || /^(rgb|#|hsl)/.test(cls)) {
                newProps.style.push({
                    color: cls
                });
            } else {
                throw new Error("style '" + cls + "' not found");
            }
        }
    }

    return newProps;
}

function recursiveStyle(elementsTree) {

    /*
     * If the node type is wrapped by tachyons then return immediately. This
     * will prevent unnecessarily applying styles to elements that have already
     * been wrapped.
     */
    if (elementsTree.type.isTachyonsWrapped) {
        return elementsTree;
    }

    var props = elementsTree.props;
    var clsPropName = _index.options.clsPropName,
        typeScale = _index.options.typeScale;

    var newProps = null;
    var translated = false;

    /* Parse cls string */
    if (_lodash2.default.isString(props[clsPropName])) {
        translated = true;
        newProps = setStyles(props, clsPropName, typeScale);
    }

    var newChildren = props.children;
    if (_lodash2.default.isArray(newChildren)) {

        /* Convert child array */
        newChildren = _react2.default.Children.toArray(newChildren);
        for (var i = 0; i < newChildren.length; i++) {
            var c = newChildren[i];
            if (_react2.default.isValidElement(c)) {
                var converted = recursiveStyle(c);
                if (converted !== c) {
                    translated = true;
                    newChildren[i] = converted;
                }
            }
        }
    } else if (_react2.default.isValidElement(newChildren)) {

        /* Convert single child */
        var _converted = recursiveStyle(newChildren);
        if (_converted !== newChildren) {
            translated = true;
            newChildren = _converted;
        }
    } else if (_lodash2.default.isFunction(newChildren)) {

        /* Convert a fumction child to evaluate on invocation */
        var originalChildrenFunction = newChildren;
        var _converted2 = function _converted2() {
            return recursiveStyle(originalChildrenFunction.apply(undefined, arguments));
        };
        if (_converted2 !== newChildren) {
            translated = true;
            newChildren = _converted2;
        }
    }

    if (translated) {
        return _react2.default.cloneElement(elementsTree, newProps, newChildren);
    }

    return elementsTree;
}